"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.merge = merge;
exports.outPackage = outPackage;
exports.getNPMFile = getNPMFile;
exports.getNPMIndex = getNPMIndex;
exports.getNPMPackage = getNPMPackage;
exports.meta = void 0;

var _path = require("path");

var _fs = require("fs");

/* babel-plugin-inline-import '../package.json' */
const config = "{\n  \"name\": \"amd-bundle\",\n  \"version\": \"1.7.8\",\n  \"description\": \"A dual-use tool for bundling up AMD modules, which is much easier to use than r.js with Almond or AMDclean\",\n  \"keywords\": [\n    \"javascript\",\n    \"amd-loader\",\n    \"bundle\",\n    \"require-js\",\n    \"command-line\"\n  ],\n  \"author\": \"shiy2008@gmail.com\",\n  \"license\": \"AGPL-3.0\",\n  \"homepage\": \"https://tech-query.me/AMD_bundle.js/\",\n  \"repository\": {\n    \"type\": \"git\",\n    \"url\": \"git+https://github.com/TechQuery/AMD_bundle.js.git\"\n  },\n  \"bugs\": {\n    \"url\": \"https://github.com/TechQuery/AMD_bundle.js/issues\"\n  },\n  \"main\": \"build/Package.js\",\n  \"bin\": \"build/index.js\",\n  \"engines\": {\n    \"node\": \"^6.3.0\"\n  },\n  \"scripts\": {\n    \"lint\": \"eslint source/ test/*.js --fix\",\n    \"bundle\": \"babel source/ -d build/ -s\",\n    \"set-test\": \"node -e \\\"console.log('exports.test = 1;')\\\" > node_modules/test.js\",\n    \"test\": \"npm run set-test  &&  mocha --exit\",\n    \"debug\": \"npm run set-test  &&  mocha --inspect-brk\",\n    \"build\": \"npm run lint  &&  npm run bundle  &&  npm test  &&  esdoc\",\n    \"prepublishOnly\": \"npm run build\",\n    \"start\": \"web-server docs/ -o\"\n  },\n  \"husky\": {\n    \"hooks\": {\n      \"pre-commit\": \"npm run build  &&  git add .\"\n    }\n  },\n  \"dependencies\": {\n    \"@babel/core\": \"^7.3.4\",\n    \"@babel/plugin-transform-modules-commonjs\": \"^7.2.0\",\n    \"@babel/polyfill\": \"^7.2.5\",\n    \"@babel/preset-env\": \"^7.3.4\",\n    \"@babel/runtime\": \"^7.3.4\",\n    \"@tech_query/node-toolkit\": \"^0.9.3\",\n    \"commander\": \"^2.19.0\",\n    \"fs-extra\": \"^7.0.1\"\n  },\n  \"devDependencies\": {\n    \"@babel/cli\": \"^7.2.3\",\n    \"@babel/plugin-transform-runtime\": \"^7.3.4\",\n    \"@babel/register\": \"^7.0.0\",\n    \"babel-plugin-inline-import\": \"^3.0.0\",\n    \"cross-env\": \"^5.2.0\",\n    \"esdoc\": \"^1.1.0\",\n    \"esdoc-external-nodejs-plugin\": \"^1.0.0\",\n    \"esdoc-standard-plugin\": \"^1.0.0\",\n    \"eslint\": \"^5.15.2\",\n    \"husky\": \"^1.3.1\",\n    \"koapache\": \"^1.0.6\",\n    \"mocha\": \"^6.0.2\",\n    \"should\": \"^13.2.3\"\n  },\n  \"amd-bundle\": {\n    \"env\": {\n      \"test\": {\n        \"moduleMap\": {\n          \"/Test/i\": \"$&4sample\"\n        }\n      }\n    }\n  }\n}\n";
/**
 * @private
 *
 * @type {Object}
 */

const meta = JSON.parse(config);
/**
 * This will be used in the bundled source
 *
 * @param {string} base - Root path
 * @param {string} path - Path relative to `base`
 *
 * @return {string} Joined & normalized path
 */

exports.meta = meta;

function merge(base, path) {
  return (base + '/' + path).replace(/\/\//g, '/').replace(/[^/.]+\/\.\.\//g, '').replace(/\.\//g, function (match, index, input) {
    return input[index - 1] === '.' ? match : '';
  });
}
/**
 * @param {string} name - Name of a module
 *
 * @return {boolean} Whether `name` is a dependency out of this package
 */


function outPackage(name) {
  return /^[^./]/.test(name);
}

function fileInNPM(path) {
  path = (0, _path.join)('node_modules', path);
  if ((0, _fs.existsSync)(path) && (0, _fs.statSync)(path).isFile()) return path.replace(/\\/g, '/');
}

function findNPMFile(list) {
  while (list[0]) {
    let path = fileInNPM(list.shift());
    if (path) return path;
  }
}
/**
 * @param {string} name - Name of a module
 *
 * @return {?string} Valid `name` in `./node_modules/` is a JS or JSON file
 */


function getNPMFile(name) {
  return findNPMFile([name, `${name}.js`, `${name}.json`]);
}
/**
 * @param {string} name - Name of a module
 *
 * @return {?string} Valid `name` in `./node_modules/` is `index.js` or `index.json`
 */


function getNPMIndex(name) {
  return findNPMFile([`${name}/index.js`, `${name}/index.json`]);
}
/**
 * @param {string} name - Name of a module
 *
 * @return {?string} Entry file path of an existing `package.json`
 */


function getNPMPackage(name) {
  var path = fileInNPM(`${name}/package.json`);
  if (!path) return;
  const config = JSON.parse((0, _fs.readFileSync)(path)),
        entry = ['main', 'browser', 'module'];

  while (entry[0]) if (path = fileInNPM((0, _path.join)(name, config[entry.shift()]))) return (0, _fs.statSync)(path).isFile() ? path : getNPMIndex(path);
}
//# sourceMappingURL=utility.js.map